@@ -1,24 +1,23 @@
 /*
- * Copyright (c) 2012 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2012, 2014 by Farsight Security, Inc.
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
- * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR
- * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
- * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
- * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 #include "mtbl-private.h"
 
-#include "librsf/heap.h"
-#include "librsf/ubuf.h"
-#include "librsf/vector.h"
+#include "libmy/heap.h"
+#include "libmy/ubuf.h"
 
 struct entry {
 	struct mtbl_iter		*it;
@@ -37,6 +36,7 @@ struct merger_iter {
 	ubuf				*cur_key;
 	ubuf				*cur_val;
 	bool				finished;
+	bool				pending;
 };
 
 struct mtbl_merger_options {
@@ -146,8 +146,6 @@ _mtbl_merger_compare(const void *va, const void *vb)
 static mtbl_res
 entry_fill(struct entry *ent)
 {
-	assert(ent->it != NULL);
-
 	const uint8_t *key, *val;
 	size_t len_key, len_val;
 	mtbl_res res;
@@ -197,8 +195,9 @@ merger_iter_next(void *v,
 
 		if (ubuf_size(it->cur_key) == 0) {
 			ubuf_clip(it->cur_val, 0);
-			ubuf_append(it->cur_key, ubuf_data(e->key), ubuf_size(e->key));
-			ubuf_append(it->cur_val, ubuf_data(e->val), ubuf_size(e->val));
+			ubuf_extend(it->cur_key, e->key);
+			ubuf_extend(it->cur_val, e->val);
+			it->pending = true;
 			res = entry_fill(e);
 			if (res == mtbl_res_success)
 				heap_replace(it->h, e);
@@ -228,12 +227,16 @@ merger_iter_next(void *v,
 		}
 	}
 
-	*out_key = ubuf_data(it->cur_key);
-	*out_val = ubuf_data(it->cur_val);
-	*out_len_key = ubuf_size(it->cur_key);
-	*out_len_val = ubuf_size(it->cur_val);
-
-	return (mtbl_res_success);
+	if (it->pending) {
+		it->pending = false;
+		*out_key = ubuf_data(it->cur_key);
+		*out_val = ubuf_data(it->cur_val);
+		*out_len_key = ubuf_size(it->cur_key);
+		*out_len_val = ubuf_size(it->cur_val);
+		return (mtbl_res_success);
+	} else {
+		return (mtbl_res_failure);
+	}
 }
 
 static void
