@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012 by Farsight Security, Inc.
+ * Copyright (c) 2012, 2014-2015 by Farsight Security, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,17 +25,43 @@ extern "C" {
 #include <stdbool.h>
 #include <stdint.h>
 
-typedef enum {
-	MTBL_COMPRESSION_NONE = 0,
-	MTBL_COMPRESSION_SNAPPY = 1,
-	MTBL_COMPRESSION_ZLIB = 2
-} mtbl_compression_type;
-
 typedef enum {
 	mtbl_res_failure = 0,
 	mtbl_res_success = 1
 } mtbl_res;
 
+/* compression */
+
+typedef enum {
+	MTBL_COMPRESSION_NONE = 0,
+	MTBL_COMPRESSION_SNAPPY = 1,
+	MTBL_COMPRESSION_ZLIB = 2,
+	MTBL_COMPRESSION_LZ4 = 3,
+	MTBL_COMPRESSION_LZ4HC = 4,
+} mtbl_compression_type;
+
+mtbl_res
+mtbl_compress(
+	mtbl_compression_type,
+	const uint8_t *input,
+	const size_t input_size,
+	uint8_t **output,
+	size_t *output_size);
+
+mtbl_res
+mtbl_decompress(
+	mtbl_compression_type,
+	const uint8_t *input,
+	const size_t input_size,
+	uint8_t **output,
+	size_t *output_size);
+
+const char *
+mtbl_compression_type_to_str(mtbl_compression_type);
+
+mtbl_res
+mtbl_compression_type_from_str(const char *, mtbl_compression_type *);
+
 /* exported types */
 
 struct mtbl_iter;
@@ -46,6 +72,7 @@ struct mtbl_reader_options;
 struct mtbl_writer;
 struct mtbl_writer_options;
 
+struct mtbl_metadata;
 struct mtbl_merger;
 struct mtbl_merger_options;
 struct mtbl_fileset;
@@ -195,6 +222,9 @@ mtbl_reader_destroy(struct mtbl_reader **);
 const struct mtbl_source *
 mtbl_reader_source(struct mtbl_reader *);
 
+const struct mtbl_metadata *
+mtbl_reader_metadata(struct mtbl_reader *);
+
 /* reader options */
 
 struct mtbl_reader_options *
@@ -203,9 +233,41 @@ mtbl_reader_options_init(void);
 void
 mtbl_reader_options_destroy(struct mtbl_reader_options **);
 
+void
+mtbl_reader_options_set_madvise_random(struct mtbl_reader_options *, bool);
+
 void
 mtbl_reader_options_set_verify_checksums(struct mtbl_reader_options *, bool);
 
+/* metadata */
+
+uint64_t
+mtbl_metadata_index_block_offset(const struct mtbl_metadata *);
+
+uint64_t
+mtbl_metadata_data_block_size(const struct mtbl_metadata *);
+
+uint64_t
+mtbl_metadata_compression_algorithm(const struct mtbl_metadata *);
+
+uint64_t
+mtbl_metadata_count_entries(const struct mtbl_metadata *);
+
+uint64_t
+mtbl_metadata_count_data_blocks(const struct mtbl_metadata *);
+
+uint64_t
+mtbl_metadata_bytes_data_blocks(const struct mtbl_metadata *);
+
+uint64_t
+mtbl_metadata_bytes_index_block(const struct mtbl_metadata *);
+
+uint64_t
+mtbl_metadata_bytes_keys(const struct mtbl_metadata *);
+
+uint64_t
+mtbl_metadata_bytes_values(const struct mtbl_metadata *);
+
 /* merger */
 
 struct mtbl_merger *
@@ -245,6 +307,9 @@ mtbl_fileset_destroy(struct mtbl_fileset **);
 void
 mtbl_fileset_reload(struct mtbl_fileset *);
 
+void
+mtbl_fileset_reload_now(struct mtbl_fileset *);
+
 const struct mtbl_source *
 mtbl_fileset_source(struct mtbl_fileset *);
 
